<?php

namespace Neutrino\Ion3Core\Models;

use App\Ion3\Models\Office;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Neutrino\Ion3Core\Database\Factories\CompanyLocationFactory;
use Neutrino\Ion3Core\Models\Ion3CompanyLocationOfficeSource;

class Ion3CompanyLocation extends Model
{
    use HasFactory;

    protected $table = 'company_locations';
    protected $companyClass = \App\Ion3\Models\Company::class;
    protected $officeClass = \App\Ion3\Models\Office::class;
    protected $companyLocationMetaClass = \App\Ion3\Models\CompanyLocationMeta::class;
    protected $userClass = \App\Models\User::class;
    protected $guarded = ['id', 'deleted_at', 'created_at', 'updated_at'];

    protected static function newFactory()
    {
        return CompanyLocationFactory::new();
    }

    public function parent(): BelongsTo
    {
        return $this->belongsTo(self::class, 'parent_id');
    }

    public function child(): HasMany
    {
        return $this->hasMany(self::class, 'parent_id');
    }

    public function company(): BelongsTo
    {
        return $this->belongsTo($this->companyClass, 'company_id');
    }

    public function offices(): BelongsToMany
    {
        return $this->belongsToMany($this->officeClass, 'offices', 'company_location_id');
    }

    public function meta(): HasMany
    {
        return $this->hasMany($this->companyLocationMetaClass, 'company_location_id');
    }

    public function users()
    {
        return $this->belongsToMany($this->userClass, 'company_location_user', 'company_location_id', 'user_id')
            ->withTimestamps();
    }

    public function office_sources(): HasMany
    {
        return $this->hasMany(Ion3CompanyLocationOfficeSource::class, 'company_location_id');
    }
}
