<?php

namespace Neutrino\Ion3Core\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Neutrino\Ion3Core\Database\Factories\ListingFactory;
use Neutrino\Ion3Core\Enums\IonLookupListingType;
use Neutrino\Ion3Core\Enums\RESOLookupPropertySubType;
use Neutrino\Ion3Core\Enums\RESOLookupPropertyType;
use Neutrino\Ion3Core\Enums\RESOLookupStandardStatus;

class Ion3Listing extends Model
{
    use HasFactory;
    protected $table = 'listings';
    protected $agentClass = \App\Ion3\Models\Agent::class;
    protected $officeClass = \App\Ion3\Models\Office::class;
    protected $listingExtraSearchFieldsClass = \App\Ion3\Models\ListingExtraSearchFields::class;
    protected $listingMediaClass = \App\Ion3\Models\ListingMedia::class;
    protected $listingMetaClass = \App\Ion3\Models\ListingMeta::class;
    protected $guarded = ['id', 'deleted_at', 'created_at', 'updated_at'];

    protected $casts = [
        'ion_listing_type' => IonLookupListingType::class,
        'property_type' => RESOLookupPropertyType::class,
        'property_sub_type' => RESOLookupPropertySubType::class,
        'standard_status' => RESOLookupStandardStatus::class,
        'lot_size_acres' => 'float',
        'internet_entire_listing_display_yn' => 'boolean',
        'internet_address_display_yn' => 'boolean',
        'latitude' => 'float',
        'longitude' => 'float',
        'modification_timestamp' => 'datetime',
        'listing_contract_date' => 'date',
        'close_date' => 'date',
        'pending_timestamp' => 'datetime',
    ];

    static function newFactory()
    {
        return ListingFactory::new();
    }

    public function extraSearchFields(): HasOne
    {
        return $this->hasOne($this->listingExtraSearchFieldsClass);
    }

    public function meta(): HasMany
    {
        return $this->hasMany($this->listingMetaClass, 'listing_id');
    }

    public function media(): HasMany
    {
        return $this->hasMany($this->listingMediaClass, 'listing_id');
    }

    public function agents(): BelongsToMany
    {
        return $this->belongsToMany($this->agentClass, 'agent_listing', 'listing_id', 'agent_id')
            ->withTimestamps()
            ->withPivot('type', 'position', 'active');
    }

    public function offices(): BelongsToMany
    {
        return $this->belongsToMany($this->officeClass, 'listing_office', 'listing_id', 'office_id')
            ->withTimestamps()
            ->withPivot('type', 'position', 'active');
    }

    public function orderedAgents()
    {
        return $this->belongsToMany($this->agentClass, 'agent_listing')
            ->wherePivot('active', 1) // Only include active agents
            ->orderByPivot('type') // Order by type in the pivot table
            ->orderByPivot('position'); // Then order by position
    }

    public function getAddress(): String
    {
        return $this->address;
    }
}
